<?php

namespace Drupal\reviews\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ReviewEntitySettingsForm.
 *
 * @ingroup reviews
 */
class ReviewEntitySettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'reviews.settings';

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'reviews_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * Constructs a new ReviewsSettingsForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['reviews_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Reviews Settings'),
    ];

    $form['reviews_settings']['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enabled'),
      '#description' => $this->t('Turn the review system on or off. When the reviews system is turned off all existing reviews data is retained but the ability to leave or view reviews is disabled.'),
      '#default_value' => $config->get('reviews.enabled'),
    ];

    $form['reviews_settings']['sort_order'] = [
      '#type' => 'select',
      '#title' => $this->t('Sort Order'),
      '#description' => $this->t('Specify the order in which to display reviews.'),
      '#options' => [
        'oldest' => $this->t('Oldest first'),
        'newest' => $this->t('Newest first'),
      ],
      '#size' => 1,
      '#default_value' => $config->get('reviews.sort_order'),
    ];

    $form['reviews_settings']['multiple'] = [
      '#type' => 'select',
      '#title' => $this->t('Multiple Reviews'),
      '#description' => $this->t('Allow a user to leave multiple reviews.'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#size' => 1,
      '#default_value' => $config->get('reviews.multiple'),
    ];

    /*

    @TO-DO
      
    $form['reviews_settings']['notify_admin'] = [
      '#type' => 'select',
      '#title' => $this->t('Notify Admin'),
      '#description' => $this->t('Notify the administrator when a review is given.'),
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
      '#size' => 1,
      '#default_value' => $config->get('reviews.notify_admin'),
    ];

    */

    $form['reviewable_types'] = [
      '#type' => 'details',
      '#title' => $this->t('Review Content Type'),
      '#open' => TRUE,
      '#caollapsible' => TRUE,
    ];

    $content_type = $this->getNodeTypes();

    $form['reviewable_types']['review_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Review Content Type'),
      '#description' => $this->t('Select the content types that you want users to be able to review.'),
      '#options' => $content_type,
      '#default_value' => $config->get('reviews.review_types'),
    ];

    $form['moderation_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Moderation Settings'),
      '#open' => FALSE,
      '#caollapsible' => TRUE,
    ];

    $form['moderation_settings']['moderate_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Do Not Moderate'),
      '#description' => $this->t('Select the content types that can be reviewed without moderation. <em>WARNING:</em> Reviews against non-moderated content types will appear published immediately.'),
      '#options' => $content_type,
      '#default_value' => $config->get('reviews.moderate_types'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $this->config(static::SETTINGS)
      ->set('reviews.enabled', $form_state->getValue('enabled'))
      ->set('reviews.sort_order', $form_state->getValue('sort_order'))
      ->set('reviews.multiple', $form_state->getValue('multiple'))
      ->set('reviews.notify_admin', $form_state->getValue('notify_admin'))
      ->set('reviews.review_types', $form_state->getValue('review_types'))
      ->set('reviews.moderate_types', $form_state->getValue('moderate_types'))
      ->save();

    // Invalidate the cache tags for the local tasks
    // and actions for this module.
    Cache::invalidateTags(['node_list']);
    Cache::invalidateTags(['reviews_action']);
  }

  /**
   * Helper function to return content type entity names.
   *
   * @return array
   *   An array containing entity names, keyed with entity type id.
   */
  private function getNodeTypes() {
    $types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    $node_types = [];

    foreach ($types as $type) {
      $node_types[$type->id()] = $type->get('name');
    }

    return $node_types;
  }

}
